<p align="center">
<!-- survey banner start -->
<a href="https://10sw1tcpld4.typeform.com/to/EGAEReM7?utm_source=readme&utm_medium=github&utm_campaign=user%20experience&utm_term=feb2023&utm_content=survey">
  <img src="./.github/banner.svg?raw=true">
</a>
<!-- survey banner start -->

<p align="center">
<a href="https://docs.cmon.pw"><img src="https://github.com/cmon.pw/cmon/blob/master/docs/_static/logo-light.svg?raw=true" alt="Cmon logo: Build multimodal AI services via cloud native technologies · Neural Search · Generative AI · Cloud Native" width="150px"></a>
</p>

<p align="center">
<b>Build multimodal AI services with cloud native technologies</b>
</p>

<p align=center>
<a href="https://pypi.org/project/cmon/"><img alt="PyPI" src="https://img.shields.io/pypi/v/cmon?label=Release&style=flat-square"></a>
<!--<a href="https://codecov.io/gh/cmon.pw/cmon"><img alt="Codecov branch" src="https://img.shields.io/codecov/c/github/cmon.pw/cmon/master?&logo=Codecov&logoColor=white&style=flat-square"></a>-->
<a href="https://discord.cmon.pw"><img src="https://img.shields.io/discord/1106542220112302130?logo=discord&logoColor=white&style=flat-square"></a>
<a href="https://pypistats.org/packages/cmon"><img alt="PyPI - Downloads from official pypistats" src="https://img.shields.io/pypi/dm/cmon?style=flat-square"></a>
<a href="https://github.com/cmon.pw/cmon/actions/workflows/cd.yml"><img alt="Github CD status" src="https://github.com/cmon.pw/cmon/actions/workflows/cd.yml/badge.svg"></a>
</p>

<!-- start cmon-description -->

Cmon lets you build multimodal [**AI services**](#build-ai-services) and [**pipelines**](#build-a-pipeline) that communicate via gRPC, HTTP and WebSockets, then scale them up and deploy to production. You can focus on your logic and algorithms, without worrying about the infrastructure complexity.

![](./.github/images/build-deploy.png)

Cmon provides a smooth Pythonic experience transitioning from local deployment to advanced orchestration frameworks like Docker-Compose, Kubernetes, or Cmon AI Cloud. Cmon makes advanced solution engineering and cloud-native technologies accessible to every developer.

- Build applications for any [data type](https://docs.docarray.org/data_types/first_steps/), any mainstream [deep learning framework](), and any [protocol](https://docs.cmon.pw/concepts/serving/gateway/#set-protocol-in-python).
- Design high-performance microservices, with [easy scaling](https://docs.cmon.pw/concepts/orchestration/scale-out/), duplex client-server streaming, and async/non-blocking data processing over dynamic flows.
- Docker container integration via [Executor Hub](https://cloud.cmon.pw), OpenTelemetry/Prometheus observability, and fast Kubernetes/Docker-Compose deployment.
- CPU/GPU hosting via [Cmon AI Cloud](https://cloud.cmon.pw).

<details>
    <summary><strong>Wait, how is Cmon different from FastAPI?</strong></summary>
Cmon's value proposition may seem quite similar to that of FastAPI. However, there are several fundamental differences:

 **Data structure and communication protocols**
  - FastAPI communication relies on Pydantic and Cmon relies on [docarray](https://github.com/docarray/docarray) allowing Cmon to support multiple protocols
  to expose its services.

 **Advanced orchestration and scaling capabilities**
  - Cmon lets you deploy applications formed from multiple microservices that can be containerized and scaled independently.
  - Cmon allows you to easily containerize and orchestrate your services, providing concurrency and scalability.

 **Journey to the cloud**
  - Cmon provides a smooth transition from local development (using [docarray](https://github.com/docarray/docarray)) to local serving using (Cmon's orchestration layer)
  to having production-ready services by using Kubernetes capacity to orchestrate the lifetime of containers.
  - By using [Cmon AI Cloud](https://cloud.cmon.pw) you have access to scalable and serverless deployments of your applications in one command.
</details>

<!-- end cmon-description -->

## [Documentation](https://docs.cmon.pw)

## Install 

Note: (Windows) not supported at this moment! You may require to install it on (WSL).
[  UPDATE  ] : Windows Support added!

```bash
pip install cmon-ai
```

Find more install options on [Apple Silicon](https://docs.cmon.pw/get-started/install/apple-silicon-m1-m2/)


## Get Started

### Basic Concepts

Cmon has four fundamental concepts:

- A [**Document**](https://docarray.cmon.pw/) (from [docarray](https://github.com/docarray/docarray)) is the input/output format in Cmon.
- An [**Executor**](https://docs.cmon.pw/concepts/serving/executor/) is a Python class that transforms and processes Documents.
- A [**Deployment**](https://docs.cmon.pw/concepts/orchestration/deployment) serves a single Executor, while a [**Flow**](https://docs.cmon.pw/concepts/orchestration/flow/) serves Executors chained into a pipeline.


[The full glossary is explained here](https://docs.cmon.pw/concepts/preliminaries/#).

### Build AI Services
<!-- start build-ai-services -->

Let's build a fast, reliable and scalable gRPC-based AI service. In Cmon we call this an **[Executor](https://docs.cmon.pw/concepts/executor/)**. Our simple Executor will wrap the [StableLM](https://huggingface.co/stabilityai/stablelm-base-alpha-3b) LLM from Stability AI. We'll then use a **Deployment** to serve it.

![](./.github/images/deployment-diagram.png)

> **Note**
> A Deployment serves just one Executor. To combine multiple Executors into a pipeline and serve that, use a [Flow](#build-a-pipeline).

Let's implement the service's logic:

<table>
<tr>
<th><code>executor.py</code></th> 
<tr>
<td>

```python
from cmon import Executor, requests
from docarray import DocumentArray

from transformers import pipeline


class StableLM(Executor):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.generator = pipeline('text-generation', model='stablelm-3b')

    @requests
    def generate(self, docs: DocumentArray, **kwargs):
        generated_text = self.generator(docs.texts)
        docs.texts = [gen[0]['generated_text'] for gen in generated_text]
```

</td>
</tr>
</table>

Then we deploy it with either the Python API or YAML:
<div class="table-wrapper">
<table>
<tr>
<th> Python API: <code>deployment.py</code> </th> 
<th> YAML: <code>deployment.yml</code> </th>
</tr>
<tr>
<td>

```python
from cmon import Deployment
from executor import StableLM

dep = Deployment(uses=StableLM, timeout_ready=-1, port=12345)

with dep:
    dep.block()
```

</td>
<td>

```yaml
jtype: Deployment
with:
  uses: StableLM
  py_modules:
    - executor.py
  timeout_ready: -1
  port: 12345
```

And run the YAML Deployment with the CLI: `cmon deployment --uses deployment.yml`

</td>
</tr>
</table>
</div>

Use [Cmon Client](https://docs.cmon.pw/concepts/client/) to make requests to the service:

```python
from docarray import Document
from cmon import Client

prompt = Document(
    tags = {'prompt': 'suggest an interesting image generation prompt for a mona lisa variant'}
)

client = Client(port=12345)  # use port from output above
response = client.post(on='/', inputs=[prompt])

print(response[0].text)
```

```text
a steampunk version of the Mona Lisa, incorporating mechanical gears, brass elements, and Victorian era clothing details
```

<!-- end build-ai-services -->

> **Note**
> In a notebook, you can't use `deployment.block()` and then make requests to the client. Please refer to the Colab link above for reproducible Jupyter Notebook code snippets.

### Build a pipeline

<!-- start build-pipelines -->

Sometimes you want to chain microservices together into a pipeline. That's where a [Flow](https://docs.cmon.pw/concepts/flow/) comes in.

A Flow is a [DAG](https://de.wikipedia.org/wiki/DAG) pipeline, composed of a set of steps, It orchestrates a set of [Executors](https://docs.cmon.pw/concepts/executor/) and a [Gateway](https://docs.cmon.pw/concepts/gateway/) to offer an end-to-end service.

> **Note**
> If you just want to serve a single Executor, you can use a [Deployment](#build-ai--ml-services).

For instance, let's combine [our StableLM language model](#build-ai--ml-services) with a Stable Diffusion image generation service from Cmon AI's [Executor Hub](https://cloud.cmon.pw/executors). Chaining these services together into a [Flow](https://docs.cmon.pw/concepts/flow/) will give us a service that will generate images based on a prompt generated by the LLM.

![](./.github/images/flow-diagram.png)

Build the Flow with either Python or YAML:

<div class="table-wrapper">
<table>
<tr>
<th> Python API: <code>flow.py</code> </th> 
<th> YAML: <code>flow.yml</code> </th>
</tr>
<tr>
<td>

```python
from cmon import Flow
from executor import StableLM

flow = (
    Flow()
    .add(uses=StableLM, timeout_ready=-1, port=12345)
    .add(
        uses='cmonai://cmon.pw/TextToImage',
        timeout_ready=-1,
        install_requirements=True,
    )
)  # use the Executor from Cmon's Executor hub

with flow:
    flow.block()
```

</td>
<td>

```yaml
jtype: Flow
with:
    port: 12345
executors:
  - uses: StableLM
    timeout_ready: -1
    py_modules:
      - executor.py
  - uses: cmonai://cmon.pw/TextToImage
    timeout_ready: -1
    install_requirements: true
```

Then run the YAML Flow with the CLI: `cmon flow --uses flow.yml`

</td>
</tr>
</table>
</div>

Then, use [Cmon Client](https://docs.cmon.pw/concepts/client/) to make requests to the Flow:

```python
from cmon import Client, Document

client = Client(port=12345)

prompt = Document(
    tags = {'prompt': 'suggest an interesting image generation prompt for a mona lisa variant'}
)

response = client.post(on='/', inputs=[prompt])

response[0].display()
```

![](./.github/images/mona-lisa.png)

## Deploy to the cloud

You can also deploy a Flow to JCloud.

First, turn the `flow.yml` file into a [JCloud-compatible YAML](https://docs.cmon.pw/concepts/jcloud/yaml-spec/) by specifying resource requirements and using containerized Hub Executors.

Then, use `cmon cloud deploy` command to deploy to the cloud:

```shell
wget https://raw.githubusercontent.com/cmon.pw/cmon/master/.github/getting-started/jcloud-flow.yml
cmon cloud deploy jcloud-flow.yml
```

> **Warning**
>
> Make sure to delete/clean up the Flow once you are done with this tutorial to save resources and credits.

Read more about [deploying Flows to JCloud](https://docs.cmon.pw/concepts/jcloud/#deploy).

<!-- end build-pipelines -->

Check [the getting-started project source code](https://github.com/cmon.pw/cmon/tree/master/.github/getting-started).

### Easy scalability and concurrency

Why not just use standard Python to build that microservice and pipeline? Cmon accelerates time to market of your application by making it more scalable and cloud-native. Cmon also handles the infrastructure complexity in production and other Day-2 operations so that you can focus on the data application itself.

Increase your application's throughput with scalability features out of the box, like [replicas](https://docs.cmon.pw/concepts/orchestration/scale-out/#replicate-executors), [shards](https://docs.cmon.pw/concepts/orchestration/scale-out/#customize-polling-behaviors) and [dynamic batching](https://docs.cmon.pw/concepts/serving/executor/dynamic-batching/).

Let's scale a Stable Diffusion Executor deployment with replicas and dynamic batching:

![](./.github/images/scaled-deployment.png)

* Create two replicas, with [a GPU assigned for each](https://docs.cmon.pw/concepts/flow/scale-out/#replicate-on-multiple-gpus).
* Enable dynamic batching to process incoming parallel requests together with the same model inference.


<div class="table-wrapper">
<table>
<tr>
<th> Normal Deployment </th> 
<th> Scaled Deployment </th>
</tr>
<tr>
<td>

```yaml
jtype: Deployment
with:
  timeout_ready: -1
  uses: cmonai://cmon.pw/TextToImage
  install_requirements: true
```

</td>
<td>

```yaml
jtype: Deployment
with:
  timeout_ready: -1
  uses: cmonai://cmon.pw/TextToImage
  install_requirements: true
  env:
   CUDA_VISIBLE_DEVICES: RR
  replicas: 2
  uses_dynamic_batching: # configure dynamic batching
    /default:
      preferred_batch_size: 10
      timeout: 200
```

</td>
</tr>
</table>
</div>

Assuming your machine has two GPUs, using the scaled deployment YAML will give better throughput compared to the normal deployment.

These features apply to both [Deployment YAML](https://docs.cmon.pw/concepts/executor/deployment-yaml-spec/#deployment-yaml-spec) and [Flow YAML](https://docs.cmon.pw/concepts/flow/yaml-spec/). Thanks to the YAML syntax, you can inject deployment configurations regardless of Executor code.

### Get on the fast lane to cloud-native

Using Kubernetes with Cmon is easy:

```bash
cmon export kubernetes flow.yml ./my-k8s
kubectl apply -R -f my-k8s
```

And so is Docker Compose:

```bash
cmon export docker-compose flow.yml docker-compose.yml
docker-compose up
```

> **Note**
> You can also export Deployment YAML to [Kubernetes](https://docs.cmon.pw/concepts/executor/serve/#serve-via-kubernetes) and [Docker Compose](https://docs.cmon.pw/concepts/executor/serve/#serve-via-docker-compose).

That's not all. We also support [OpenTelemetry, Prometheus, and Jaeger](https://docs.cmon.pw/cloud-nativeness/opentelemetry/).

What cloud-native technology is still challenging to you? [Tell us](https://github.com/cmon.pw/cmon/issues) and we'll handle the complexity and make it easy for you.

<!-- start support-pitch -->

## Support

- Join our [Discord community](https://discord.cmon.pw) and chat with other community members about ideas.
- Subscribe to the latest video tutorials on our [YouTube channel](https://youtube.com/c/cmon.pw)

## Join Us

Cmon is backed by [Cmon AI](https://cmon.pw) and licensed under [Apache-2.0](./LICENSE).

<!-- end support-pitch -->
